package {
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.MouseEvent;
	import flash.external.ExternalInterface;
	
	import it.h_umus.osc.OSCArgument;
	import it.h_umus.osc.OSCConnection;
	import it.h_umus.osc.OSCMessage;
	import it.h_umus.osc.OSCPacket;
	
	import org.wiiflash.Wiimote;
	import org.wiiflash.events.WiimoteEvent;
	
	import uranodai.oscemote.IPhoneas;
	import uranodai.oscemote.event.OSCemoteTouchEvent;
	import uranodai.oscemote.model.touch.TouchData;
	
	[SWF(width="320",height="240",frameRate="30",backgroundColor="#FFFFFF")]
	
	/**
	 * ストリートビューとiPhoneとバランスWiiボードでバーチャル東京マラソン。
	 */
	public class Main extends Sprite
	{
		private var connection:OSCConnection;
		private var wiimote:Wiimote;
		private var iphone:IPhoneas;
		private var sview:StreetViewManager;
		
		private var host:String = "localhost";
		private var port:int = 3000;
		private var sendPort:int = 5500;
		
		private var stepCount:int;
		private var touched:Boolean = false;
		private var tempx:Number;
		private var tempy:Number;
		private var currentPitch:Number;
		private var currentYaw:Number;
		
		public function Main()
		{
			// iPhoneasオブジェクトを生成します。
			iphone = new IPhoneas(host, port);
			iphone.addEventListener(OSCemoteTouchEvent.TOUCH, onTouch);
			// flosc Serverに接続します
			iphone.connect();
			
			// OSCConnectionオブジェクトを用意します。
			connection = iphone.connection;
						
			// Wiimoteオブジェクトを生成します。
			wiimote = new Wiimote();
			
			// 各イベントリスナを追加します。
			wiimote.addEventListener(Event.CONNECT, trace);
			wiimote.addEventListener(IOErrorEvent.IO_ERROR, trace);
			wiimote.addEventListener(WiimoteEvent.BALANCEBOARD_CONNECT, trace);
			wiimote.addEventListener(WiimoteEvent.BALANCEBOARD_DISCONNECT, trace);
			wiimote.addEventListener(WiimoteEvent.UPDATE, onUpdate);
			
			// WiiFlash Serverに接続します。
			wiimote.connect();
			
			// StreetViewManagerオブジェクトを生成します。
			sview = new StreetViewManager();
			
			// 歩数を初期化します。
			stepCount = 0;
			
			// for debug
			stage.addEventListener(MouseEvent.CLICK, function(e:MouseEvent):void
			{
				proceedStep();
			});
		}
		
		/**
		 * iPhoneタッチイベントのリスナ
		 */
		private function onTouch(e:OSCemoteTouchEvent):void
		{
			if(e.count==1)
			{
				var t:TouchData = e.getTouch(0);
				
				if(!touched)
				{
					touched = true;
					tempx = t.x;
					tempy = t.y;
					
					// ストリートビューを更新します
					if(!ExternalInterface.available) return;
					currentYaw = sview.getCurrentYaw();
					currentPitch = sview.getCurrentPitch();
					// 向きを変更します。
					sview.pan(currentYaw, currentPitch);
				}
				else
				{
					var cx:Number = t.x - tempx;
					var cy:Number = t.y - tempy;
					var yaw:Number = cx*120;
					var pitch:Number = cy*120;
					
					yaw+=currentYaw;
					pitch+=currentPitch;
					
					// ストリートビューを更新します
					if(!ExternalInterface.available) return;
					// 向きを変更します。
					sview.pan(yaw, pitch);
				}
					
				
			}
			else if(e.count==0 && touched)
			{
				touched = false;
				//trace("touch end");	
			}
		}
		
		/**
		 *  バランスWiiボードの各センサの値を処理します。
		 */
		private function onUpdate(e:WiimoteEvent):void
		{
			// 重量が3kg未満の場合はスルーします。
			if(wiimote.balanceBoard.totalKg < 3) return;
			
			var leftKg:Number = wiimote.balanceBoard.topLeftKg + wiimote.balanceBoard.bottomLeftKg;
			var rightKg:Number = wiimote.balanceBoard.topRightKg + wiimote.balanceBoard.bottomRightKg;
			
			// 左足が浮いたとき、歩数を進めます。
			if(stepCount%2==0 && leftKg/rightKg>2)
			{
				proceedStep();
			}
			// 右足が浮いたとき、歩数を進めます。
			else if(stepCount%2==1 && rightKg/leftKg>2)
			{
				proceedStep();
			}
		}
		
		/**
		 * 歩数を進めます。
		 */
		private function proceedStep():void
		{
			// 歩数を更新します。
			stepCount++;
			
			trace(this,"proceedStep", stepCount);
			
			// chumby を更新します。
			if(!connection.connected) return;
			
			// OSCパケットを作成します。
			var p:OSCPacket = new OSCPacket();
			p.address = host;
			p.port = sendPort;
			var msg:OSCMessage = new OSCMessage("/tokyo_marathon/step");
			msg.addArg(new OSCArgument(OSCArgument.i, stepCount));
			p.addMessage(msg);
			
			// OSCパケットを送信します。
			connection.sendOSCPacket(p);
			
			// ストリートビューを更新します
			if(!ExternalInterface.available) return;
			// 前進します
			sview.goFront();
		}
	}
}
